----------------
-- Window for Challenge
----------------

local AceGUI = LibStub("AceGUI-3.0")

-- Lua APIs
local pairs, assert, type = pairs, assert, type
local tblsort,tblremove = table.sort, table.remove

-- WoW APIs
local PlaySound = PlaySound
local CreateFrame, UIParent = CreateFrame, UIParent
local C_TimerAfter = C_Timer.After

----------------
-- Main Frame --
----------------
--[[
    Events :

]]
do
	local Type = "GoldGoblinLootGUIContainer-SortedScrollFrame"
	local Version = 1

	local function OnAcquire(self)
        self:ScrollFrameOnAcquire()
        self:SetDescending()
        self.descending = nil
	end

	local function OnRelease(self)
        self:ScrollFrameOnRelease()
        self:SetRefreshInterval()
    end

    local function SorterFunction(a, b)
        return a:GetSortValue() > b:GetSortValue()
    end

    local function SorterFunctionDescanding(a, b)
        return a:GetSortValue() < b:GetSortValue()
    end

    local function SetRefreshInterval(self, interval)
        -- set new interval
        self.interval = interval
        if not self.isRunning then
            self._OnRefresh()
        end
    end

    local function SetDescending(self, descending)
        self.descending = descending
    end

    local function SortChildrenList(self)
        tblsort(self.children, self.descending and SorterFunctionDescanding or SorterFunction)

        -- update children
        for i = 1, #self.children do
            if self.children[i].UpdateListRank then
                self.children[i]:UpdateListRank(i, true)
            end
        end

        self:DoLayout()

        self:Fire("OnListSort")
    end

    local function ReleaseSingleChildren(self, childrenWidget)
        local children = self.children
        for i = 1,#children do
            if children[i] == childrenWidget then
                AceGUI:Release(children[i])
                tblremove(children, i)
                return
            end
		end
    end

    local function Constructor()
        -- use 'Window' as base
        local scrollFrame = AceGUI:Create("ScrollFrame")

        local self = scrollFrame
        local frame = scrollFrame.frame
		self.type = Type

        -- save and overwirte
        self.ScrollFrameOnRelease = self.OnRelease
		self.ScrollFrameOnAcquire = self.OnAcquire
		self.OnRelease = OnRelease
        self.OnAcquire = OnAcquire
        self.SetDescending = SetDescending

        -- own things
        self.SortChildrenList = SortChildrenList
        self.SetRefreshInterval = SetRefreshInterval
        self.ReleaseSingleChildren = ReleaseSingleChildren

        -- refresh function
        self._OnRefresh = function()
            if not self.interval then
                self.isRunning = nil
                return
            end
            self:SortChildrenList()
            self.isRunning = true
            C_TimerAfter(self.interval, self._OnRefresh)
        end

		AceGUI:RegisterAsContainer(self)
		return self
	end

	AceGUI:RegisterWidgetType(Type,Constructor,Version)
end
