local AddonName, GGL = ...
local GUI = GGL.GUI
local Global = {}
GGL.Options.Invitations = Global

local L = GGL.Locale

-- lua
local pairs, ipairs = pairs, ipairs

-- WoW

-- Libs
local AceGUI = LibStub("AceGUI-3.0")

-- ##################
-- local
-- ##################
local LoadedFrame
local SessionIDLines = {}
local GUISave = {}

local STATUS = GGL.INVITE_KEY_LIST

-- ##################
-- local helper
-- ##################

local function UpdateSessionStatus()
    if not GUISave.leaveSession then return end

    local lockedSession = GGL.Session:GetLockedSession()

    if lockedSession then
        if lockedSession:PlayerIsHost() then
            GUISave.statusText:SetText("|cffdaa205 "..L["You are currently host of a session."].."|r")
            GUISave.leaveSession:SetDisabled(true)
        else
            GUISave.statusText:SetText("|cff4dbf3d "..L["You are currently participant in a session."].."|r")
            GUISave.leaveSession:SetDisabled(false)
        end
    else
        GUISave.statusText:SetText("|cffdd3e3e "..L["No session running."].."|r")
        GUISave.leaveSession:SetDisabled(true)
    end
end

-- leaveSession OnClick
local function LeaveSession_OnClick(self)
    if not GUISave.leaveSession then return end
    if not GGL.Session:GetLockedSession() then return end

    local lockedSession = GGL.Session:GetLockedSession()
    -- also decline session
    lockedSession:GetInviteTemplate():SetOwnInviteState(GGL.INVITE_KEY_LIST.DECLINED)
    -- unlock session
    GGL.Session:UnlockSession()
end


local refreshInvitesSpamProtect
local function RefreshInvites_OnClick(self)
    if refreshInvitesSpamProtect and (GetTime() - refreshInvitesSpamProtect) < 2 then return end
    GGL.Invite:RequestInvites(true)
    refreshInvitesSpamProtect = GetTime()
end

--- updates the state list of all invitations
local function UpdateInvitationList()
    if not LoadedFrame then return end
    local children = LoadedFrame.children
    for index, child in ipairs(children) do
        -- child.sessionType
    end
end

--- removes a invitation
local function RemoveInviteEntry(inviteTemplate)
    if not LoadedFrame then return end
    local sessionID = inviteTemplate:GetSessionID()
    local children = LoadedFrame.children
    local numChildern = #children
    if numChildern > 0 then
        local fix = 0
        for i = 1, numChildern do
            local child = children[i]
            children[i] = nil

            if child.sessionID == sessionID then
                child:Release()
                fix = fix + 1
            else
                --child:SetCount(i-fix)
                children[i-fix] = child
            end
        end
        LoadedFrame:DoLayout()
    end
    UpdateInvitationList()
    GGL.Session:UnlockSession(inviteTemplate.session)
end

--- OnClick from accept/decline buttons
local function StateButton_OnClick(self, event, oldState, inviteTemplate)
    local sessionID = inviteTemplate:GetSessionID()
    local sessionType = inviteTemplate:GetSessionType()

    if event == "OnInviteAccept" then
        -- if there is a session from the same type accepted decline the other
        if GGL.Session:GetLockedSession() and GGL.Session:GetLockedSession() ~= inviteTemplate:GetSession() then
            LeaveSession_OnClick()
        end

        -- set new player state for inviteTemplate
        inviteTemplate:SetOwnInviteState(STATUS.ACCEPTED)

        -- lock session
        GGL.Session:LockSession(inviteTemplate:GetSession())

        -- decline all other invites
        local inviteList = GGL.Invite:GetInvitationList()
        for listInviteTemplate in pairs(inviteList) do
            if listInviteTemplate ~= inviteTemplate then
                listInviteTemplate:SetOwnInviteState(STATUS.DECLINED)
            end
        end
    elseif event == "OnInviteDecline" then
        -- when the session is the locked one unlock it
        if GGL.Session:GetLockedSession() and GGL.Session:GetLockedSession() == inviteTemplate:GetSession() then
            LeaveSession_OnClick()
        else
            inviteTemplate:SetOwnInviteState(STATUS.DECLINED)
        end
    end
    UpdateInvitationList()
end

--- adds a new inviteTemplate into the GUI
local function AddInviteEntry(inviteTemplate)
    if not LoadedFrame then return end
    local children = LoadedFrame.children
    local numChildern = #children
    if numChildern > 0 then
        for i = 1, numChildern do
            if children[i].sessionID == inviteTemplate:GetSessionID() then
                return
            end
        end
    end

    local line = AceGUI:Create("GoldGoblinLootGUIWidget-InvitationLine")
    line:SetFullWidth(true)
    line:SetInviteTemplate(inviteTemplate)
    line:SetCallback("OnInviteAccept", StateButton_OnClick)
    line:SetCallback("OnInviteDecline", StateButton_OnClick)
    LoadedFrame:AddChild(line)

    SessionIDLines[inviteTemplate:GetSessionID()] = line

    -- decline invite when one is accepted
    if GGL.Session:GetLockedSession() and GGL.Session:GetLockedSession() ~= inviteTemplate:GetSession() then
        inviteTemplate:SetOwnInviteState(STATUS.DECLINED)
    end
end

--- loads all open invitations in the list
local function FillInvitationList()
    local inviteList = GGL.Invite:GetInvitationList()
    for inviteTemplate in pairs(inviteList) do
        AddInviteEntry(inviteTemplate)
    end
end

-- ## Callbacks
local function CB_Invite_NewInvite(inviteTemplate, description)
    if not LoadedFrame then return end
    AddInviteEntry(inviteTemplate)
end

local function CB_Invite_InviteRemoved(inviteTemplate)
    if not LoadedFrame then return end
    RemoveInviteEntry(inviteTemplate)
end

local function WidgetOnRelease(self, event)
    LoadedFrame = nil
    wipe(SessionIDLines)
    wipe(GUISave)
end

-- ##################
-- options
-- ##################
local function Options(AceGUI, container)

    local fillGroup = AceGUI:Create("SimpleGroup")
    fillGroup:SetFullWidth(true)
    fillGroup:SetLayout("List")
    container:AddChild(fillGroup)

    local topGroup = AceGUI:Create("SimpleGroup")
    topGroup:SetFullWidth(true)
    topGroup:SetLayout("Flow")
    topGroup:SetHeight(50)
    fillGroup:AddChild(topGroup)

    local description = AceGUI:Create("Label")
    description:SetFullWidth(true)
    description:SetFontObject(GameFontHighlight)
    description:SetJustifyH("CENTER")
    description:SetJustifyV("MIDDLE")
    description:SetText(L["Please enter your Twitch Name under Settings before participating in a challenge.\nBy default your battle tag is displayed."])
    topGroup:AddChild(description)

    local line1 = AceGUI:Create("Heading")
    line1:SetFullWidth(true)
    topGroup:AddChild(line1)

    local statusText = AceGUI:Create("Label")
    statusText:SetFullWidth(true)
    statusText:SetFontObject(GameFontHighlightLarge)
    GUISave.statusText = statusText
    topGroup:AddChild(statusText)

    local line2 = AceGUI:Create("Heading")
    line2:SetFullWidth(true)
    topGroup:AddChild(line2)

    local leaveSession = AceGUI:Create("Button")
    leaveSession:SetText(L["Leave"])
    leaveSession:SetAutoWidth(true)
    leaveSession:SetCallback("OnClick", LeaveSession_OnClick)
    GUISave.leaveSession = leaveSession
    topGroup:AddChild(leaveSession)

    local refreshInvites = AceGUI:Create("Button")
    refreshInvites:SetText(L["Refresh invites"])
    refreshInvites:SetAutoWidth(true)
    refreshInvites:SetCallback("OnClick", RefreshInvites_OnClick)
    GUISave.refreshInvites = refreshInvites
    topGroup:AddChild(refreshInvites)

    local line3 = AceGUI:Create("Heading")
    line3:SetFullWidth(true)
    topGroup:AddChild(line3)

    local scroll = AceGUI:Create("ScrollFrame")
    scroll:SetLayout("List")
    scroll:SetFullWidth(true)
    scroll:SetHeight(container.content:GetHeight() - topGroup.frame:GetHeight() - 10)
    scroll:SetCallback("OnRelease", WidgetOnRelease)
    fillGroup:AddChild(scroll)

    -- set needed things
    LoadedFrame = scroll

    -- init
    FillInvitationList()
    UpdateInvitationList()
    UpdateSessionStatus()
end

-- Add into Options
GGL.Options:AddIntoTree(Options, L["Challenge Invites"], "invitations")

-- cb register
GGL.Invite:RegisterCallbackEvent("NewInvite", CB_Invite_NewInvite)
GGL.Invite:RegisterCallbackEvent("InviteRemoved", CB_Invite_InviteRemoved)
GGL.Session:RegisterCallbackEvent("OnSessionLocked", UpdateSessionStatus)
GGL.Session:RegisterCallbackEvent("OnSessionLockReleased", UpdateSessionStatus)