local AddonName, GGL = ...
local GUI = GGL.GUI
local Global = {}
GGL.Options.Session = Global

local L = GGL.Locale

-- lua
local pairs = pairs

-- WoW

-- Libs
local AceGUI = LibStub("AceGUI-3.0")

-- ##################
-- local
-- ##################


-- ##################
-- local helper
-- ##################


-- ##################
-- options
-- ##################

local function TimeEdit_OnEnterPressed(self, event, text)
    text = tonumber(text)
    if not text then
        self:SetText(GGL.db.Options.duration)
    else
        if text > 6060 then
            text = 6060
            self:SetText(text)
        end
        GGL.db.Options.duration = text
    end
    self:ClearFocus()
end

local function SetDropDown_OnValueChanged(self, event, value)
    GGL.db.Options.set = value or "Base"
end

local function GetSetDropDownList()
    local db = GGL.db.Config.Sets[GGL.SET_ID_LIST.Set]

    local list = {}
    for k,v in pairs(db) do
        list[k] = v.name
    end

    return list
end

local function StartSession_OnClick(self, event, mb)
    -- TODO: Add new Session start
end

local function EndSession_OnClick(self, event, mb)
    local session = GGL.Templates.Session:GetRunningSession()
    if session then
        session:Close()
    end
end

local function Options(AceGUI, container)
    local db = GGL.db.Options

    local scroll = AceGUI:Create("ScrollFrame")
    scroll:SetLayout("Flow")
    container:AddChild(scroll)

    local top = AceGUI:Create("SimpleGroup")
    top:SetFullWidth(true)
    top:SetLayout("Flow")
    scroll:AddChild(top)

    local time = AceGUI:Create("EditBox")
    time:SetWidth(80)
    time:SetLabel(L["Time"])
    time:SetText(db.duration)
    time:SetMaxLetters(4)
    time:SetCallback("OnEnterPressed", TimeEdit_OnEnterPressed)
    top:AddChild(time)

    local timeMin = AceGUI:Create("Label")
    timeMin:SetText(L["min"])
    top:AddChild(timeMin)

    local setDropdown = AceGUI:Create("Dropdown")
    setDropdown:SetList(GetSetDropDownList())
    setDropdown:SetValue(db.set)
    setDropdown:SetCallback("OnValueChanged", SetDropDown_OnValueChanged)
    top:AddChild(setDropdown)

    local bottom = AceGUI:Create("SimpleGroup")
    bottom:SetFullWidth(true)
    bottom:SetLayout("Flow")
    scroll:AddChild(bottom)

    local startSession = AceGUI:Create("Button")
    startSession:SetText(L["Start session"])
    startSession:SetAutoWidth(true)
    startSession:SetCallback("OnClick", StartSession_OnClick)
    bottom:AddChild(startSession)

    local endSession = AceGUI:Create("Button")
    endSession:SetText(L["End running session"])
    endSession:SetAutoWidth(true)
    endSession:SetCallback("OnClick", EndSession_OnClick)
    bottom:AddChild(endSession)
end

-- Add into Options
GGL.Options:AddIntoTree(Options, L["Solo"], "session")