local AddonName, GGL = ...
local GUI = GGL.GUI
local Sets = GGL.Options.Sets
local Global = {}
GGL.Options.Sets_Item = Global

local L = GGL.Locale

-- lua

-- WoW

-- Libs
local AceGUI = LibStub("AceGUI-3.0")

-- ##################
-- local
-- ##################
local Helper


-- ##################
-- local helper
-- ##################

-- ##################
-- options
-- ##################

local function SourceDropDown_OnValueChanged(self, event, key)
    local data = Helper.Data
    local db = data["selectedDB"]
    local qualityID = self:GetUserData("qualityID")

    if key == GGL.PriceSource:GetDefaultID() then
        db[qualityID] = nil
    else
        db[qualityID] = key
    end

    Helper:SaveDB()
end

local function RawGoldCheck_OnValueChanged(self, event, state)
    local data = Helper.Data
    local db = data["selectedDB"]

    if state == false then
        db["r"] = true
    else
        db["r"] = nil
    end

    Helper:SaveDB()
    Helper:RefreshSelectedTab()
end

local function SoulboundCheck_OnValueChanged(self, event, state)
    local data = Helper.Data
    local db = data["selectedDB"]

    if state == false then
        db["s"] = nil
    else
        db["s"] = db["s"] or GGL.PriceSource:GetDefaultID()
    end

    Helper:SaveDB()
    Helper:RefreshSelectedTab()
end

local function SetOnTabGroupConfigSelected(content, db, tabID)
    local base = AceGUI:Create("SimpleGroup")
    base:SetFullWidth(true)
    base:SetLayout("Flow")

    -- RawGold
    local rgCheck = AceGUI:Create("CheckBox")
    rgCheck:SetLabel(L["Raw gold"])
    rgCheck:SetImage(237281)
    if db["r"] == true then
        rgCheck:SetValue(false)
    else
        rgCheck:SetValue(true)
    end
    rgCheck:SetCallback("OnValueChanged", RawGoldCheck_OnValueChanged)
    base:AddChild(rgCheck)

    local tLine = AceGUI:Create("Heading")
    tLine:SetFullWidth(true)
    base:AddChild(tLine)

    -- Soulbound
    local sbCheck = AceGUI:Create("CheckBox")
    sbCheck:SetLabel(ITEM_SOULBOUND)
    sbCheck:SetImage(132853)
    sbCheck:SetValue(db["s"] and true or false)
    sbCheck:SetCallback("OnValueChanged", SoulboundCheck_OnValueChanged)
    base:AddChild(sbCheck)

    -- soulbound items
    if db["s"] then
        local sbDropdown = AceGUI:Create("Dropdown")
        local sbList, sbSortList = GGL.PriceSource:GetDropDownListData()
        sbDropdown:SetList(sbList, sbSortList)
        sbDropdown:SetValue(db["s"] or GGL.PriceSource:GetDefaultID())
        sbDropdown:SetCallback("OnValueChanged", SourceDropDown_OnValueChanged)
        sbDropdown:SetUserData("qualityID", "s")
        base:AddChild(sbDropdown)
    end

    for qualityID = 0, #GGL.ITEM_QUALITY_INFO do
        local quality = GGL.ITEM_QUALITY_INFO[qualityID]

        local line = AceGUI:Create("Heading")
        line:SetFullWidth(true)
        base:AddChild(line)

        local text = AceGUI:Create("Label")
        text:SetText(quality.coloredDesc)
        base:AddChild(text)

        local dropdown = AceGUI:Create("Dropdown")
        local list, sortList = GGL.PriceSource:GetDropDownListData()
        dropdown:SetList(list, sortList)
        dropdown:SetValue(db[qualityID] or GGL.PriceSource:GetDefaultID())
        dropdown:SetCallback("OnValueChanged", SourceDropDown_OnValueChanged)
        dropdown:SetUserData("qualityID", qualityID)
        base:AddChild(dropdown)
    end
    local line = AceGUI:Create("Heading")
    line:SetFullWidth(true)
    base:AddChild(line)

    content:AddChild(base)
end

local function Options(AceGUI, container)
    Helper = Sets:GetBaseGroup(container, GGL.SET_ID_LIST.PriceSource)

    -- empty for now
    local tabs = {}

    --Helper:SetOnTabGroupSelected(SetOnTabGroupSelected)
    Helper:SetOnTabGroupConfigSelected(SetOnTabGroupConfigSelected)

    Helper:SetTabGroupContent(tabs, true)
end

-- Add into Options
GGL.Options:AddIntoTree(Options, L["Price Source"], "sets_pricesource", "sets")