local AddonName, GGL = ...
local GUI = GGL.GUI
local Global = {}
GGL.Options.Settings = Global

local L = GGL.Locale

-- lua

-- WoW

-- Libs
local AceGUI = LibStub("AceGUI-3.0")

-- ##################
-- local
-- ##################

-- ##################
-- local helper
-- ##################
local function SetSettingsDbValue(category, key, value)
    if GGL.db.Settings[category] then
        GGL.db.Settings[category][key] = value

        return true
    end
    return false
end

local function GetSettingsDbValue(category, key)
    return GGL.db.Settings[category] and GGL.db.Settings[category][key] or nil
end

-- ##################
-- local
-- ##################
local function WidgetOnRelease(self, event)

end

-- ##################
-- Settings - Top items
-- ##################
local TopItemsDbHandler

local function TopItems_NumTopItems_OnValueChanged(self, event, newValue)
    TopItemsDbHandler:Set("numTopItems", newValue)
end

local function TopItems_MinGoldValue_OnValueChanged(self, event, text)
    text = tonumber(text or 0)
    if text and text > 0 then
        text = text * 10000
        text = text > 99999999999 and 99999999999 or text
        TopItemsDbHandler:Set("minGoldValue", text)
        self:SetText(text / 10000)
    else
        TopItemsDbHandler:Set("minGoldValue", 0)
        self:SetText(0)
    end
    self:ClearFocus()
end

local function Settings_TopItems(AceGUI, container)
    TopItemsDbHandler = TopItemsDbHandler or GGL.Options:GetDbGetterAndSetter(GGL.db, "Settings", "TopItems")

    local scroll = AceGUI:Create("ScrollFrame")
    scroll:SetLayout("Flow")
    scroll:SetCallback("OnRelease", WidgetOnRelease)
    container:AddChild(scroll)

    local numTopItems = AceGUI:Create("Slider")
    numTopItems:SetFullWidth(true)
    numTopItems:SetSliderValues(1, GGL.MAX_TOP_ITEMS, 1)
    numTopItems:SetLabel(L["Maximum items shown."])
    numTopItems:SetValue(TopItemsDbHandler:Get("numTopItems"))
    numTopItems:SetCallback("OnValueChanged", TopItems_NumTopItems_OnValueChanged)
    scroll:AddChild(numTopItems)

    local minGoldValue = AceGUI:Create("EditBox")
    minGoldValue:SetFullWidth(true)
    minGoldValue:SetLabel(L["Minimum item gold value"])
    minGoldValue:SetText(TopItemsDbHandler:Get("minGoldValue") / 10000)
    minGoldValue:SetMaxLetters(7)
    minGoldValue:SetCallback("OnEnterPressed", TopItems_MinGoldValue_OnValueChanged)
    GGL.GUI:AddToolTipForAceGUIWidget(minGoldValue, L["Minimum item gold value"], L["For running session this only affects new drops."], L["A reload of the TopList can be forced with a reload of the session."])
    scroll:AddChild(minGoldValue)
end

-- Add into Options
GGL.Options:AddIntoTree(Settings_TopItems, L["Top Items"], "settings_topitems")


-- ##################
-- Settings
-- ##################

local function Name_OnEnterPressed(self, event, text)
    text = GGL.FormatDisplayName(text)
    GGL.db.Settings.Main.displayName = text
    GGL.Templates.BNWoWPlayer:GetSelfPlayer():SetDisplayName(text)
    self:SetText(text)
    self:ClearFocus()
end

local function ShowMinimapButton_OnValueChanged(self, event, state)
    GGL:MMB_SetShownState(state)
end

local function GglAlertAnchor_OnClick(self, mb)
    GGL.LootAlert:UnlockAlertFrameAnchor(nil)
end

local function Settings_Options(AceGUI, container)
    local scroll = AceGUI:Create("ScrollFrame")
    scroll:SetLayout("Flow")
    scroll:SetCallback("OnRelease", WidgetOnRelease)
    container:AddChild(scroll)

    local name = AceGUI:Create("EditBox")
    name:SetFullWidth(true)
    name:SetLabel(L["Name"])
    name:SetText(GGL.Templates.BNWoWPlayer:GetSelfPlayer().displayName)
    name:SetMaxLetters(GGL.MAX_NAME_LENGTH)
    name:SetCallback("OnEnterPressed", Name_OnEnterPressed)
    scroll:AddChild(name)

    local line1 = AceGUI:Create("Heading")
    line1:SetText(L["Minimap Button"])
    line1:SetFullWidth(true)
    scroll:AddChild(line1)

    local showMinimapButton = AceGUI:Create("CheckBox")
    showMinimapButton:SetFullWidth(true)
    --:SetRelativeWidth(0.5)
    showMinimapButton:SetLabel(SHOW)
    showMinimapButton:SetValue(GGL.db.MinimapButton.shown)
    showMinimapButton:SetCallback("OnValueChanged", ShowMinimapButton_OnValueChanged)
    scroll:AddChild(showMinimapButton)

    local line2 = AceGUI:Create("Heading")
    line2:SetText(L["GGL Loot Alert Anchor"])
    line2:SetFullWidth(true)
    scroll:AddChild(line2)

    local gglAlertAnchor = AceGUI:Create("Button")
    gglAlertAnchor:SetText(L["Show / Hide"])
    gglAlertAnchor:SetFullWidth(true)
    gglAlertAnchor:SetCallback("OnClick", GglAlertAnchor_OnClick)
    scroll:AddChild(gglAlertAnchor)

    -- resize scrollFrame
    scroll:DoLayout()
end

-- Add into Options
GGL.Options:AddIntoTree(Settings_Options, L["Settings"], "settings")

-- ##################
-- Settings - Streamer header
-- ##################
local StreamerDbHandler

local function Streamer_BattleTag_OnValueChanged(self, event, text)
    if not text or text == "" then
        local _, battleTag = BNGetInfo()
        StreamerDbHandler:Set("battleTag", nil)
        self:SetText(battleTag)
    else
        StreamerDbHandler:Set("battleTag", text)
    end
    self:ClearFocus()
end

local function Streamer_Title_OnValueChanged(self, event, text)
    StreamerDbHandler:Set("title", text)
    self:ClearFocus()
end

local function Streamer_Check_OnValueChanged(self, event, state)
    StreamerDbHandler:Set(self:GetUserData("key"), state)
end

local function Settings_Streamer(AceGUI, container)
    StreamerDbHandler = StreamerDbHandler or GGL.Options:GetDbGetterAndSetter(GGL.db, "Settings", "Streamer")

    local _, battleTag = BNGetInfo()

    local scroll = AceGUI:Create("ScrollFrame")
    scroll:SetLayout("Flow")
    scroll:SetCallback("OnRelease", WidgetOnRelease)
    container:AddChild(scroll)

    local enabled = AceGUI:Create("CheckBox")
    enabled:SetFullWidth(true)
    enabled:SetLabel(ENABLE)
    enabled:SetValue(StreamerDbHandler:Get("enabled"))
    enabled:SetCallback("OnValueChanged", Streamer_Check_OnValueChanged)
    enabled:SetUserData("key", "enabled")
    scroll:AddChild(enabled)

    --[[
    local hideBanner = AceGUI:Create("CheckBox")
    hideBanner:SetFullWidth(true)
    hideBanner:SetLabel("Hide banner")
    hideBanner:SetValue(StreamerDbHandler:Get("hideBanner"))
    hideBanner:SetCallback("OnValueChanged", Streamer_Check_OnValueChanged)
    hideBanner:SetUserData("key", "hideBanner")
    scroll:AddChild(hideBanner)
    ]]

    local title = AceGUI:Create("EditBox")
    title:SetFullWidth(true)
    title:SetLabel(L["Title"])
    title:SetText(StreamerDbHandler:Get("title"))
    title:SetCallback("OnEnterPressed", Streamer_Title_OnValueChanged)
    scroll:AddChild(title)

    local battleTagEB = AceGUI:Create("EditBox")
    battleTagEB:SetFullWidth(true)
    battleTagEB:SetLabel(L["Battle tag"])
    battleTagEB:SetText(StreamerDbHandler:Get("battleTag") or battleTag)
    battleTagEB:SetCallback("OnEnterPressed", Streamer_BattleTag_OnValueChanged)
    scroll:AddChild(battleTagEB)
end

-- Add into Options
--GGL.Options:AddIntoTree(Settings_Streamer, L["Streamer"], "settings_streamer", "settings")