-- ##################
--- EVENTS ( GGL.Templates.BNWoWPlayer:RegisterCallbackEvent )
-- "NewPlayerRegistered" (playerTemplate)
-- "PlayerUpdate" (playerTemplate)
-- "DisplayNameUpdate" (playerTemplate, displayName)
-- "AccountIDUpdate" (playerTemplate, newAccountID, oldAccountID)
-- "MergeIDUpdate" (playerTemplate, newMergeID, oldMergeID)
-- ##################

local AddonName, GGL = ...
local Templates = GGL.Templates

local BNWoWPlayer, BNWoWPlayerHelper = Templates:Register("BNWoWPlayer")

-- lua
local assert, type = assert, type
local pairs, next = pairs, next
local match, format = string.match, string.format

-- wow
local BNGetGameAccountInfoByGUID, BNGetAccountInfoByGUID = C_BattleNet.GetGameAccountInfoByGUID, C_BattleNet.GetAccountInfoByGUID

local WOW_PROJECT_ID = _G.WOW_PROJECT_ID

-- ##################
-- local
-- ##################
local MERGE_ID_FORMAT = "%s!%d"

local DummyGUIDRegister = {}
local GUIDRegister = {}
local SpecialRegister = {}
local PlayerSelf

-- ##################
-- static
-- ##################
local Static = {}
Templates.BNWoWPlayer = Static

function Static:Get(unitGUID)
    return unitGUID and GUIDRegister[unitGUID] or nil
end

function Static:GetByGameAccountID(gameAccountID)
    for guid, player in pairs(GUIDRegister) do
        if player.gameAccountID == gameAccountID and player.isOnline then
            return player
        end
    end
end

function Static:GetByGUID(unitGUID)
    if unitGUID and GUIDRegister[unitGUID] then
        return GUIDRegister[unitGUID]
    end
end

function Static:GetPlayerSender()
    if not self.playerSender then
        local sender = C_BattleNet.GetGameAccountInfoByGUID(UnitGUID("player"))
        if sender then
            self.playerSender = sender.gameAccountID
        end
    end
    return self.playerSender
end

-- method(BNWoWPlayer)
function Static:RegisterCallbackEvent(...)
    BNWoWPlayerHelper:RegisterCallbackEvent(...)
end

function Static:UnregisterCallbackEvent(...)
    BNWoWPlayerHelper:UnregisterCallbackEvent(...)
end

-- for playerGUID, player in Templates.BNWoWPlayer:IterPlayerList() do end
function Static:IterPlayerList()
    return next, GUIDRegister, nil
end

local function EmptyDummyFunction() end

function Static:GetDummyPlayer(syncID)
    if DummyGUIDRegister[syncID] then
        return DummyGUIDRegister[syncID]
    else
        local battleTag, unitGUID = match(syncID, "^(.*):(.*)")
        if not battleTag then return end

        -- create a new player entry if its not existing or we get errors
        if not GGL.db.PlayerRegister[syncID] then
            GGL.db.PlayerRegister[syncID] = {}
        end

        local _, ownBattleTag = BNGetInfo()
        local playerGUID = UnitGUID("player")

        local dummy = {
            -- block some calls here
            Constructor = EmptyDummyFunction,
            Update = EmptyDummyFunction,
            RegisterCallbackEvent = EmptyDummyFunction,
            UnregisterCallbackEvent = EmptyDummyFunction,
            FireCallbackEvent = EmptyDummyFunction,
            -- some needed things
            syncID = syncID,
            gglVersion = "dummy",
            isOnline = false,
            unitGUID = unitGUID,
            isPlayerBattleTag = ownBattleTag == battleTag,
            isPlayer = playerGUID == unitGUID,
            bnPlayer = {
                battleTag = battleTag,
            },
            displayName = GGL.db.PlayerRegister[syncID].displayName or syncID,
            -- detection that its a dummy
            isDummy = true,
        }
        setmetatable(dummy, {__index = BNWoWPlayer})
        dummy:UpdateMergeID()
        DummyGUIDRegister[syncID] = dummy

        return dummy
    end
end

-- Creates a template of the player
function Static:GetSelfPlayer()
    if not PlayerSelf then
        if not Templates.BNPlayer:GetSelf() then return end
        PlayerSelf = Templates:New("BNWoWPlayer", UnitGUID("player"), Templates.BNPlayer:GetSelf())
        PlayerSelf:SetDisplayName(GGL.db.Settings.Main.displayName)
    end
    return PlayerSelf
end

-- ##################
-- template
-- ##################
function BNWoWPlayer:Constructor(unitGUID, bnPlayer)
    assert(unitGUID, "No 'unitGUID' found ("..(unitGUID or "nil")..")")
    assert(bnPlayer, "No 'bnPlayer' found.")
    if GUIDRegister[unitGUID] then
        return GUIDRegister[unitGUID]
    end

    local isPlayer
    local playerGUID = UnitGUID("player")
    --check if guid is players guid
    if playerGUID == unitGUID then
        isPlayer = true
    end

    local bnetAccount = BNGetAccountInfoByGUID(unitGUID)
    -- maybe support non Bnetaccs later?
    if not isPlayer and not bnetAccount then return false end

    local gameAccount = BNGetGameAccountInfoByGUID(unitGUID)
    if not isPlayer and not gameAccount then return false end

    -- check if given can be used
    if isPlayer and bnetAccount and gameAccount then
        isPlayer = false
    end

    local _, ownBattleTag = BNGetInfo()

    -- battle.net account
    self.unitGUID = unitGUID
    self.bnetAccountID = isPlayer and (bnetAccount and bnetAccount.bnetAccountID or 0) or bnetAccount.bnetAccountID

    -- game account
    self.gameAccountID = isPlayer and (gameAccount and gameAccount.gameAccountID or 0) or gameAccount.gameAccountID
    -- self.realmName = gameAccount.realmName -- gets name later in game... use realmID
    self.isPlayer = playerGUID == unitGUID
    self.isPlayerBattleTag = ownBattleTag == bnPlayer.battleTag
    self.realmID = isPlayer and GetRealmID() or gameAccount.realmID
    self.characterName = isPlayer and UnitName("player") or gameAccount.characterName
    self.factionName = isPlayer and UnitFactionGroup("player") or gameAccount.factionName
    self.raceName = isPlayer and UnitRace("player") or gameAccount.raceName
    self.className = isPlayer and UnitClass("player") or gameAccount.className
    self.bnPlayer = bnPlayer
    self.syncID = bnPlayer.battleTag..":"..unitGUID
    self.sender = self.gameAccountID

    GUIDRegister[unitGUID] = self

    if not GGL.db.PlayerRegister[self.syncID] then
        GGL.db.PlayerRegister[self.syncID] = {}
    end

    -- remove dummy if its there
    DummyGUIDRegister[self.syncID] = nil

    -- add own callback handler
    GGL:AddCallbackHandler(self)

    self:UpdateMergeID()
    self:Update()

    BNWoWPlayerHelper:FireCallbackEvent("NewPlayerRegistered", self)
end

function BNWoWPlayer:Update()
    local gameAccount = BNGetGameAccountInfoByGUID(self.unitGUID)

    if not gameAccount then
        -- offline
        self.isOnline = false
    else
        if gameAccount then
            self.isGameAFK = gameAccount.isGameAFK
            self.richPresence = gameAccount.richPresence -- "Dazar'alor - Blackrock"
            self.characterLevel = gameAccount.characterLevel
            self.areaName = gameAccount.areaName
            self.isWowMobile = gameAccount.isWowMobile
            self.isOnline = gameAccount.isOnline
            self.realmName = gameAccount.realmName or "realmID:"..self.realmID -- realmName gets loaded later after login but is there after a reload/refresh
        end
        self.fullName = self.characterName.."-"..self.realmName

        self:RefreshDisplayName()
    end

    self:FireCallbackEvent("PlayerUpdate", self)
    BNWoWPlayerHelper:FireCallbackEvent("PlayerUpdate", self)
end

--- Updates the displayName for the player
function BNWoWPlayer:RefreshDisplayName()
    self.displayName = GGL.db.PlayerRegister[self.syncID].displayName or self.bnPlayer.battleTag
end

--- Sets the display name for a player
-- @param       displayName         <string>
function BNWoWPlayer:SetDisplayName(displayName)
    displayName = displayName and GGL.FormatDisplayName(displayName) or nil

    if not displayName or displayName == "" then
        -- set default to battle.tag
        displayName = self.bnPlayer.battleTag
    end

    -- update db for that player
    GGL.db.PlayerRegister[self.syncID].displayName = displayName == "" and nil or displayName

    -- update player
    self:Update()

    -- callbacks
    self:FireCallbackEvent("DisplayNameUpdate", self, displayName)
    BNWoWPlayerHelper:FireCallbackEvent("DisplayNameUpdate", self, displayName)
end

--- Returns displayName for a player
function BNWoWPlayer:GetDisplayName()
    return GGL.db.PlayerRegister[self.syncID].displayName or self.syncID
end

function BNWoWPlayer:SetGGLData(version)
    self.gglVersion = version
end

function BNWoWPlayer:GetDBTable()
    return GGL.db.PlayerRegister[self.syncID]
end

function BNWoWPlayer:UpdateMergeID()
    local newMergeID
    if self.isPlayerBattleTag then
        newMergeID = "self"
    elseif GGL.db.PlayerRegister[self.syncID].accountID then
        newMergeID = format(MERGE_ID_FORMAT, self.bnPlayer.battleTag, GGL.db.PlayerRegister[self.syncID].accountID)
    else
        newMergeID = 0
    end
    if self.mergeID ~= newMergeID then
        local oldMergeID = self.mergeID
        self.mergeID = newMergeID
        self:FireCallbackEvent("MergeIDUpdate", self, newMergeID, oldMergeID)
    end
end

--- NEVER call this directly!!!
function BNWoWPlayer:SetAccountID(accountID)
    local oldAccountID = GGL.db.PlayerRegister[self.syncID].accountID
    if oldAccountID == accountID then return false end
    GGL.db.PlayerRegister[self.syncID].accountID = accountID
    self:UpdateMergeID()
    self:FireCallbackEvent("AccountIDUpdate", self, accountID, oldAccountID)
    return true
end

function BNWoWPlayer:GetAccountID()
    return GGL.db.PlayerRegister[self.syncID].accountID
end

function BNWoWPlayer:CanSendMessage(ignoreGGL)
    if not ignoreGGL and not self.gglVersion then
        return false
    elseif self.isOnline then
        return true
    end
end

function BNWoWPlayer:SetTimeOffset(offset)
    if offset == 0 then offset = nil end
    GGL.db.PlayerRegister[self.syncID].timeOffset = offset
end

function BNWoWPlayer:GetTimeOffset()
    return GGL.db.PlayerRegister[self.syncID].timeOffset or 0
end