local AddonName, GGL = ...
local Templates = GGL.Templates

local PriceSource = GGL.PriceSource

local ItemPriceSet, ItemPriceSetHelper = Templates:Register("ItemPriceSet")

-- lua

-- wow
local GetItemInfo = GetItemInfo
local ITEM_QUALITY_COLORS = _G.ITEM_QUALITY_COLORS


-- ##################
-- local
-- ##################
local NUM_QUALITYS = #ITEM_QUALITY_COLORS

-- ##################
-- static
-- ##################
local Static = {}
Templates.ItemPriceSet = Static

-- ##################
-- template
-- ##################
function ItemPriceSet:Constructor(setKey, isSettingString)
    -- set baisc settings with default price source
    if isSettingString then
        self:LoadSetSettings(setKey)
    else
        self:LoadSet(setKey)
    end
end

function ItemPriceSet:LoadSet(setKey)
    local db = GGL.db.Config.Sets[GGL.SET_ID_LIST.PriceSource]
    self.setKey = setKey
    if db and db[setKey] then
        local pass, db = GGL:Deserialize(db[setKey].setting)
        self:SetFilterDB(db)
    else
        self:SetFilterDB()
    end
end

function ItemPriceSet:LoadSetSettings(setting)
    if setting then
        local pass, db = GGL:Deserialize(setting)
        self:SetFilterDB(db)
    else
        self:SetFilterDB()
    end
end

function ItemPriceSet:ReloadSet()
    if self.setKey then
        self:LoadSet(self.setKey)
    end
end

function ItemPriceSet:SetFilterDB(db)
    db = db or {}
    self.db = db
    local price = {}
    for qualityID = 0, #GGL.ITEM_QUALITY_INFO do
        local quality = GGL.ITEM_QUALITY_INFO[qualityID]
        price[qualityID] = GGL.PriceSource:GetFunction(db[qualityID])
    end
    if db["s"] then
        price["s"] = GGL.PriceSource:GetFunction(db["s"])
    end
    self.price = price
end

--- Returns the price in copper
function ItemPriceSet:GetPrice(itemID)
    local _, _, quality, _, _, _, _, _, _, _, _, _, _, bindType = GetItemInfo(itemID)
    -- not cached
    if not quality then
        return nil
    end

    local price = self.price

    if bindType == 1 then
        if price["s"] then
            return price["s"](itemID)
        else
            return 0
        end
    end

    return price[quality](itemID)
end

function ItemPriceSet:RawGoldAllowed()
    return self.db == true and false or true
end