local AddonName, GGL = ...
local Templates = GGL.Templates
local Sender = GGL.Sender
local Session = Templates.Session

local Multibox, MultiboxHelper = Templates:Register("Multibox", "Session", "Sync")

-- lua

-- wow

-- ##################
-- local
-- ##################
local SessionSave = {}
local INVITE_STATES = GGL.INVITE_KEY_LIST

-- ##################
-- static
-- ##################
local Static = {}
Templates.Multibox = Static

function Static:GetSession(sessionDB, sessionID)
    sessionDB = sessionDB or GGL.sessionDb.Sessions
    if sessionID and SessionSave[sessionID] then
        return SessionSave[sessionID], false
    else
        return Templates:New("Multibox", sessionDB, sessionID), true
    end
end

-- ##################
-- template
-- EVENTS
--      "AddSecondItem" (sessionSelf, itemID, sourceID, ammount, copperValue, mainSourceFlag, subSourceFlag, sourcePlayer, mainListAdded)
-- ##################
Multibox.sessionFormat = "M%d"
Multibox.sessionType = "M"

local function CB_HostStartsSession(self, session, duration, endTime, settingsString)
    session:SetData("Base", "Base", true)
    session:SetTimerData(endTime - duration, endTime)
    session:Start()
end

local function CB_HostEndsSession(self, session)
    session:Close()
end

local function CB_OnDescriptionChange(self, description)
    if not self:PlayerIsHost() and description and description == GGL.db.MultiboxOptions.autoAcceptKey then
        self:SetOwnInviteState(INVITE_STATES.ACCEPTED)
    end
end

function Multibox:Constructor(sessionDB, sessionID)
    if sessionID and SessionSave[sessionID] then
        return SessionSave[sessionID]
    end
    self:CallBaseFunction("Session", "Constructor", sessionDB, sessionID)

    -- set auto accept key as description
    self:SetDescription(GGL.db.MultiboxOptions.autoAcceptKey)

    self:CallBaseFunction("Sync", "Constructor")

    self.inviteTemplate:RegisterCallbackEvent("HostStartsSession", CB_HostStartsSession)
    self.inviteTemplate:RegisterCallbackEvent("HostEndsSession", CB_HostEndsSession)
    self.inviteTemplate:RegisterCallbackEvent("OnDescriptionChange", CB_OnDescriptionChange)

    SessionSave[self.db.sessionID] = self
end

local function CB_MFrameOnNewItem(self, itemID, zoneID, sourceID, ammount, copperValue, mainSourceFlag, subSourceFlag, sourcePlayer)
    if not self.multiboxFrame then return end
    self.multiboxFrame:AddItem(itemID, ammount, copperValue, sourcePlayer)
    self.multiboxFrame:SetValueGold(copperValue * ammount, true)
    self.multiboxFrame:SetPlayerGold(sourcePlayer, copperValue * ammount, true)
end

local function CB_MFrameOnAddMoney(self, copper, sourcePlayer)
    if not self.multiboxFrame then return end
    self.multiboxFrame:SetRawGold(copper, true)
    self.multiboxFrame:SetPlayerGold(sourcePlayer, copper, true)
end

local function CB_MFrameOnAddSecondItem(self, itemID, sourceID, ammount, copperValue, mainSourceFlag, subSourceFlag, sourcePlayer, mainListAdded)
    if not self.multiboxFrame then return end
    self.multiboxFrame:AddItemSpecial(itemID, ammount, copperValue, sourcePlayer)
    if not mainListAdded then
        self.multiboxFrame:SetValueGold(copperValue * ammount, true)
        self.multiboxFrame:SetPlayerGold(sourcePlayer, copperValue * ammount, true)
        -- add also into main frame with second filter
        self:MainFrameOnNewItem(itemID, sourceID, ammount, copperValue, mainSourceFlag, subSourceFlag, sourcePlayer)
    end
end

local function CB_MFrameOnNewSyncLoot(self, sourcePlayer, lootTime, zoneIniID, difficultyID, itemID, ammount, sourceID, mainSourceFlag, subSourceFlag)
    self.pauseLog = true
    if itemID == 0 then
        self:AddMoney(ammount, nil, sourceID, mainSourceFlag, subSourceFlag, nil, zoneIniID, difficultyID, lootTime, nil, sourcePlayer)
    else
        self:AddNewItem(itemID, ammount, nil, sourceID, mainSourceFlag, subSourceFlag, nil, zoneIniID, difficultyID, lootTime, nil, sourcePlayer)
    end
    self.pauseLog = nil
end

local function CB_MFrameOnPlayerAccepted(self, player)
    if self.multiboxFrame then
        self.multiboxFrame:AddPlayer(player)
    end
end

local function CB_MFrameOnPlayerDeclined(self, player)

end

function Multibox:AddNewItem(itemID, ammount, source, sourceID, mainSourceFlag, subSourceFlag, categoryFlag, zoneID, difficultyID, lootTime, lootLog, sourcePlayer)
    local mainListAdded = self:CallBaseFunction("Session", "AddNewItem", itemID, ammount, source, sourceID, mainSourceFlag, subSourceFlag, categoryFlag, zoneID, difficultyID, lootTime, lootLog, sourcePlayer)
    if not self.dataIsSet then return end
    local db = self.db

    -- get loot zone
    zoneID = zoneID or self.zoneID
    difficultyID = difficultyID or self.zoneDifficultyID

    sourcePlayer = sourcePlayer or self.selfPlayer

    -- check is is valid
    if (self.SecondItemFilter and self.SecondItemFilter:IsValidItem(itemID)) and (not self.ZoneFilter or self.ZoneFilter:IsValidZone(zoneID, difficultyID)) then
        if source then
            source = Session:FormatSource(source)
            for k, v in pairs(source) do
                if not self.LootSourceFilter or self.LootSourceFilter:IsValidSource(k or mainSourceFlag, subSourceFlag) then
                    self:FireCallbackEvent("AddSecondItem", self, itemID, sourceID, v, db.priceLog[itemID], k or mainSourceFlag, subSourceFlag, sourcePlayer, mainListAdded)
                end
            end
        else
            if not self.LootSourceFilter or self.LootSourceFilter:IsValidSource(mainSourceFlag, subSourceFlag) then
                self:FireCallbackEvent("AddSecondItem", self, itemID, sourceID, ammount, db.priceLog[itemID], mainSourceFlag, subSourceFlag, sourcePlayer, mainListAdded)
            end
        end
    end
end

function Multibox:RemoveFromMultiBoxFrame()
    if not self.multiboxFrame then return end
    local frame = self.multiboxFrame

    frame.contentContainer.timer:FinishTimer()

    frame.runningSession = nil

    frame:SetUpdateInterval(nil)

    -- remove from minimap button
    GGL.GUI.Multibox:RemoveFromMiniMapButton()

    -- callback
    self:UnregisterCallbackEvent("AddItem", CB_MFrameOnNewItem)
    self:UnregisterCallbackEvent("AddMoney", CB_MFrameOnAddMoney)
    self:UnregisterCallbackEvent("AddSecondItem", CB_MFrameOnAddSecondItem)
    self:UnregisterCallbackEvent("NewSyncLoot", CB_MFrameOnNewSyncLoot)
    self:UnregisterCallbackEvent("PlayerAccepted", CB_MFrameOnPlayerAccepted)
    self:UnregisterCallbackEvent("PlayerDeclined" , CB_MFrameOnPlayerDeclined)
end

function Multibox:AddIntoMultiBoxFrame(add)
    if not self.db or self.db.isClosed then return end
    if add then
        local frame = GGL.GUI.Multibox.frame or GGL.GUI.Multibox:CreateFrame()
        -- end session that is loaded into main frame
        if frame and frame.runningSession and frame.runningSession ~= self then
            frame.runningSession:Close()
        end
        local db = self.db
        -- start timer if its a running session or just set timer
        if db.isRunning then
            frame.contentContainer.timer:SetTime(db.startTime, db.endTime)
            frame.contentContainer.timer:IsCounter(false)
            frame.contentContainer.timer:StartTimer()
        elseif db.durationTime then
            frame.contentContainer.timer:SetDuration(db.durationTime)
        end

        -- set data
        if self.multiboxFrame ~= frame then
            self.multiboxFrame = frame
            frame.runningSession = self

            -- clear all
            frame:Clear()
            self.rebuildSync = nil

            -- set chars
            --:GetInvitedPlayerList()
            frame:AddPlayer(self.selfPlayer)
            local playerList = self.playerList
            for player, inviteState in self:IterAcceptedPlayers() do
                frame:AddPlayer(player)
            end

            -- update
            frame:SetUpdateInterval(15)

            -- add into minimap button
            GGL.GUI.Multibox:AddIntoMiniMapButton()

            -- callback
            self:RegisterCallbackEvent("AddItem", CB_MFrameOnNewItem)
            self:RegisterCallbackEvent("AddMoney", CB_MFrameOnAddMoney)
            self:RegisterCallbackEvent("AddSecondItem", CB_MFrameOnAddSecondItem)
            self:RegisterCallbackEvent("NewSyncLoot", CB_MFrameOnNewSyncLoot)
            self:RegisterCallbackEvent("PlayerAccepted", CB_MFrameOnPlayerAccepted)
            self:RegisterCallbackEvent("PlayerDeclined" , CB_MFrameOnPlayerDeclined)
        end
    else
        self:RemoveFromMultiBoxFrame()
    end
end

function Multibox:Start()
    if self.multiboxFrame then
        self:AddIntoMultiBoxFrame(true)
    end

    self:CallBaseFunction("Session", "Start")
    self:CallBaseFunction("Sync", "Start")
    if not self.db.isRunning then return end

    if self.multiboxFrame then
        self:AddIntoMultiBoxFrame(true)
        GGL.GUI.Multibox:Show()
    end

    return self.db.startTime, self.db.endTime, self.db.durationTime
end

function Multibox:SetData(mainSet, secondItemFilter, setRefresh)
    self:CallBaseFunction("Session", "SetData", mainSet, setRefresh)

    local db = self.db

    -- set secondItemFilter
    if db.setSave then
        if not db.setSave.secondItemFilter or setRefresh then
            db.setSave.secondItemFilter = GGL.db.Config.Sets[GGL.SET_ID_LIST.Item][GGL.db.MultiboxOptions.secondItemFilter].setting
        end

        if setRefresh and self.SecondItemFilter then
            self.SecondItemFilter:LoadSetSettings(db.setSave.secondItemFilter)
        else
            self.SecondItemFilter = Templates:New("ItemFilter", db.setSave.secondItemFilter, true)
        end
    end

    if self.multiboxFrame then
        self:AddIntoMultiBoxFrame(true)
    end
end

local function UnregisterAll(self)
    self:RemoveFromMultiBoxFrame()

    SessionSave[self.db.sessionID] = nil
end

function Multibox:Unregister()
    self:CallBaseFunction("Session", "Unregister")
    self:CallBaseFunction("Sync", "Unregister")

    UnregisterAll(self)
end

function Multibox:Stop()
    self:CallBaseFunction("Session", "Stop")
    self:CallBaseFunction("Sync", "Stop")

    UnregisterAll(self)

    if not self.inviteTemplate then return end
    self.inviteTemplate:RegisterCallbackEvent("HostStartsSession", CB_HostStartsSession)
    self.inviteTemplate:RegisterCallbackEvent("HostEndsSession", CB_HostEndsSession)
    self.inviteTemplate:RegisterCallbackEvent("OnDescriptionChange", CB_OnDescriptionChange)
    SessionSave[self.db.sessionID] = self
end

--- Close the session
function Multibox:Close(notArchiv)
    self:CallBaseFunction("Session", "Close", notArchiv)
    self:CallBaseFunction("Sync", "Close", notArchiv)

    UnregisterAll(self)
end